#include "MMKeys.h"
#include <string.h>
#include <nspr/prthread.h>
#include <string/nsEmbedString.h>

static Display *dpy = NULL;
static Window root;
static Window win;
static int screen;

static  std::list<MMKeyEntry*> g_keys;

static KeySym grab_key(char *opt)
{
  KeySym keysym;

  keysym = XStringToKeysym(opt);
  int key_code = XKeysymToKeycode(dpy, keysym);

  XGrabKey (dpy, key_code, 0, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod2Mask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod5Mask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, LockMask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod2Mask | Mod5Mask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod2Mask | LockMask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod5Mask | LockMask, root, True, GrabModeAsync, GrabModeAsync);
  XGrabKey (dpy, key_code, Mod2Mask | Mod5Mask | LockMask, root, True, GrabModeAsync, GrabModeAsync);

  return keysym;
}

static void key_main(void* dummy) {
  XKeyEvent event;

  while(1) {
    XNextEvent(dpy, (XEvent*)&event);

    if(event.type==KeyPress) {
      KeySym sym = XKeycodeToKeysym(dpy, event.keycode, 0);

  	std::list<MMKeyEntry*>::iterator iter;
  	for (iter = g_keys.begin(); iter != g_keys.end(); iter++)
  	{
    		MMKeyEntry *entry = *iter;
    		if (entry->m_keysym==sym) {
			entry->m_callback->HandleKey(entry->m_key);
			break;
		}
  	}
//      printf("foo: %s\n", key_name); 
    }
  }
}

static int thread_running = 0;

NS_IMPL_ISUPPORTS1(MMKeys, IMMKeys)

MMKeys::MMKeys()
{
  if(dpy==NULL) {
    if (!(dpy = XOpenDisplay(NULL))) {
      fprintf(stderr, " can not open dpy %s", XDisplayName(NULL));
    }
    screen = DefaultScreen(dpy);
    root = RootWindow(dpy, screen);
  }
}

MMKeys::~MMKeys()
{
}

/* void addKey (in AString key, in IMMKeyCallback callback); */
NS_IMETHODIMP MMKeys::AddKey(const nsAString & key, IMMKeyCallback *callback)
{
   nsEmbedCString keyStr("");

   NS_UTF16ToCString(key, NS_CSTRING_ENCODING_UTF8, keyStr); 
   const char* str;
    int len = NS_CStringGetData(keyStr, &str);
    char* buffer = (char*)malloc(len+1);

    bzero(buffer, len+1);
    strncpy(buffer, str, len);
    
    MMKeyEntry* entry = new MMKeyEntry();
    entry->m_key = key;
    NS_ADDREF(callback);
    entry->m_callback = callback;
    
    entry->m_keysym = grab_key(buffer);

    g_keys.push_back(entry);

    if(!thread_running) {
      PR_CreateThread(PR_USER_THREAD, key_main, NULL, PR_PRIORITY_NORMAL, PR_LOCAL_THREAD, PR_JOINABLE_THREAD, 0);
      thread_running = 1;
    }

    free(buffer);

    return NS_OK;
}
