/***********************************************************
constants
***********************************************************/

// reference to the interface 
const IMMKeysPlugin = Components.interfaces.IMMKeysPlugin;
const IMMKeyCallback = Components.interfaces.IMMKeyCallback;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;
const nsIObserver = Components.interfaces.nsIObserver;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{1abf4486-7d8f-43eb-88a3-0a7c6a1a4395}");

// description
const CLASS_NAME = "MMKeys plugin";

// textual unique identifier
const CONTRACT_ID = "@danielparnell.com/MMKeysPlugin;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function MMKeysPlugin() {
};

function _TimerCallback(key) {
  this._key = key;
}
_TimerCallback.prototype = {
  _key: "",
  QueryInterface: function(iid) {
    if (iid.Equals(Components.interfaces.nsITimerCallback) || iid.Equals(Components.interfaces.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  notify: function(timer) {
	var playback = Components.classes["@songbirdnest.com/Songbird/PlaylistPlayback;1"]
		.getService(Components.interfaces.sbIPlaylistPlayback);
	
	switch(this._key) {
	case 'XF86AudioPlay':
		if(playback.playing && !playback.paused) {
			playback.pause();
		} else {
			playback.play();
		}
		break;
	case 'XF86AudioStop':
		playback.stop();
		break;
	case 'XF86AudioNext':
		playback.next();
		break;
	case 'XF86AudioPrev':
		playback.previous();
		break;
	}
  }
};

// class definition
MMKeysPlugin.prototype = {
  timer: null,

  handleKey: function(key) {
  	this.timer.initWithCallback(new _TimerCallback(key), 0, this.timer.TYPE_ONE_SHOT);
  },

  // watch for XRE startup and shutdown messages 
  observe: function(subject, topic, data) {
    switch (topic) {
    case 'app-startup':
      // create a timer to allow the back end thread to notify the UI that we want to do things
      this.timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
      var factory = Components.classes['@danielparnell.com/Songbird/MMKeys;1'];

      if(factory) {
      	var mmkeys = factory.createInstance(Components.interfaces.IMMKeys);

	mmkeys.addKey('XF86AudioPlay', this);
      	mmkeys.addKey('XF86AudioStop', this);
      	mmkeys.addKey('XF86AudioNext', this);
      	mmkeys.addKey('XF86AudioPrev', this);
      } else {
        dump("Could not create the MMKeys instance\n");
      }
      break;
    }
  },

  QueryInterface: function(aIID)
  {
    if (!aIID.equals(IMMKeysPlugin) &&
	!aIID.equals(IMMKeyCallback) &&
	!aIID.equals(nsIObserver) &&
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myHelloWorld = Components.classes["@dietrich.ganx4.com/helloworld;1"].
                          createInstance(Components.interfaces.nsIHelloWorld);

***********************************************************/
var MMKeysPluginFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new MMKeysPlugin()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var MMKeysPluginModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);

    var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
                                    .getService(Components.interfaces.nsICategoryManager);
    categoryManager.addCategoryEntry("app-startup", CLASS_NAME,
                                     CONTRACT_ID, 
                                     true, true, null);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        

    var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
                                    .getService(Components.interfaces.nsICategoryManager);
    categoryManager.deleteCategoryEntry("app-startup",
                                        CONTRACT_ID, true);

  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return MMKeysPluginFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return MMKeysPluginModule; }